@@ -3,11 +3,13 @@
 ;; Copyright (C) 2016-2020 Syohei YOSHIDA and Neil Okamoto
 
 ;; Author: Syohei YOSHIDA <syohex@gmail.com>
-;; Maintainer: Neil Okamoto <neil.okamoto+melpa@gmail.com>
-;; URL: https://github.com/emacsorphanage/anzu
-;; Version: 0.64
+;; Maintainer: LemonBreezes <look@strawberrytea.xyz>
+;; Homepage: https://github.com/emacsorphanage/anzu
+;; Version: 0.66
 ;; Package-Requires: ((emacs "25.1"))
 
+;; This file is part of GNU Emacs.
+
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation, either version 3 of the License, or
@@ -25,10 +27,10 @@
 
 ;; `anzu.el' is an Emacs port of `anzu.vim'.
 ;;
-;; `anzu.el' provides a minor mode which displays 'current match/total
-;; matches' in the mode-line in various search modes.  This makes it
-;; easy to understand how many matches there are in the current buffer
-;; for your search query.
+;; `anzu.el' provides a minor mode which displays the match count for
+;; various search commands in the mode-line, using the format
+;; `current/total'.  This makes it easy to know how many matches your
+;; search query has in the current buffer.
 
 ;; To use this package, add following code to your init.el or .emacs
 ;;
@@ -41,23 +43,24 @@
 (require 'thingatpt)
 
 (defgroup anzu nil
-  "Show searched position in mode-line"
+  "Show the current search's match count in the mode-line."
   :group 'isearch)
 
 (defcustom anzu-mode-lighter " Anzu"
-  "Lighter of anzu-mode"
+  "Mode-line lighter for `anzu-mode'."
   :type 'string)
 
 (defcustom anzu-cons-mode-line-p t
-  "Set nil if you use your own mode-line setting"
+  "Whether anzu should display itself in the mode-line.
+Set to nil if you put anzu in your mode-line manually."
   :type 'boolean)
 
 (defcustom anzu-minimum-input-length 1
-  "Minimum input length to enable anzu"
+  "Minimum search query length required to enable anzu."
   :type 'integer)
 
 (defcustom anzu-search-threshold 1000
-  "Limit of search number"
+  "Search match count limit."
   :type '(choice (integer :tag "Threshold of search")
                  (const :tag "No threshold" nil)))
 
@@ -67,45 +70,45 @@
                  (const :tag "No threshold" nil)))
 
 (defcustom anzu-use-migemo nil
-  "Flag of using migemo"
+  "Whether to use migemo."
   :type 'boolean)
 
 (defcustom anzu-mode-line-update-function #'anzu--update-mode-line-default
-  "Function which return mode-line string. This must be non-nil."
+  "Function which returns the mode-line string.  This must be non-nil."
   :type 'function)
 
 (defcustom anzu-regexp-search-commands '(isearch-forward-regexp
                                          isearch-backward-regexp)
-  "Search function which use regexp."
+  "Search functions which use regexp."
   :type '(repeat function))
 
 (defcustom anzu-input-idle-delay 0.05
-  "Idle second for updating modeline at replace commands"
+  "Delay in seconds between mode-line updates in replace commands."
   :type 'number)
 
 (defcustom anzu-deactivate-region nil
-  "Deactive region if you use anzu a replace command with region"
+  "Whether to deactivate region when anzu is used with a region replace command."
   :type 'boolean)
 
 (defcustom anzu-replace-at-cursor-thing 'defun
-  "Replace thing. This parameter is same as `thing-at-point'"
+  "Thing to replace.  See `thing-at-point' for valid options."
   :type 'symbol)
 
 (defcustom anzu-replace-to-string-separator ""
-  "Separator of `to' string"
+  "Separator of `to' string."
   :type 'string)
 
 (defface anzu-mode-line
   '((t (:foreground "magenta" :weight bold)))
-  "face of anzu modeline")
+  "Anzu's mode-line indicator face.")
 
 (defface anzu-mode-line-no-match
   '((t (:inherit anzu-mode-line)))
-  "face of anzu modeline in no match case")
+  "Anzu's mode-line indicator face, used when no matches are found.")
 
 (defface anzu-replace-highlight
   '((t :inherit query-replace))
-  "highlight of replaced string")
+  "Replacement highlighting face.")
 
 (defface anzu-match-1
   '((((class color) (background light))
@@ -136,7 +139,7 @@
      :foreground "red")
     (((class color) (background dark))
      :foreground "yellow"))
-  "highlight of replace string")
+  "Replacement highlighting face.")
 
 (defvar anzu--total-matched 0)
 (defvar anzu--current-position 0)
@@ -153,6 +156,7 @@
 (defvar anzu--history nil)
 (defvar anzu--query-defaults nil)
 (defvar anzu--region-noncontiguous nil)
+(defvar anzu--update-timer nil)
 
 (defun anzu--validate-regexp (regexp)
   (condition-case nil
@@ -266,7 +270,7 @@
 
 (defun anzu--cons-mode-line (state)
   (setq anzu--state state)
-  (when (and anzu-cons-mode-line-p (not (anzu--mode-line-not-set-p)))
+  (when (and anzu-cons-mode-line-p (listp mode-line-format) (not (anzu--mode-line-not-set-p)))
     (setq mode-line-format (cons anzu--mode-line-format mode-line-format))))
 
 (defsubst anzu--reset-status ()
@@ -306,7 +310,7 @@
 
 ;;;###autoload
 (define-minor-mode anzu-mode
-  "minor-mode which display search information in mode-line."
+  "Minor mode which displays the current search's match count in the mode-line."
   :init-value nil
   :global     nil
   :lighter    anzu-mode-lighter
@@ -387,12 +391,24 @@
                thereis (and (>= beg b overlay-beg) (<= end e overlay-end)))
     (and (>= beg overlay-beg) (<= end overlay-end))))
 
+(defun anzu--convert-for-lax-whitespace (str use-regexp)
+  (if use-regexp
+      (if replace-regexp-lax-whitespace
+          (replace-regexp-in-string "\\s-+" search-whitespace-regexp str
+                                    nil t)
+        str)
+    (if replace-lax-whitespace
+        (replace-regexp-in-string "\\s-+"
+                                  search-whitespace-regexp
+                                  (regexp-quote str)
+                                  nil t)
+      (regexp-quote str))))
+
 ;; Return highlighted count
 (defun anzu--count-and-highlight-matched (buf str replace-beg replace-end
                                               use-regexp overlay-limit case-sensitive)
   (anzu--cleanup-markers)
-  (when (not use-regexp)
-    (setq str (regexp-quote str)))
+  (setq str (anzu--convert-for-lax-whitespace str use-regexp))
   (if (not (anzu--validate-regexp str))
       anzu--cached-count
     (with-current-buffer buf
@@ -496,25 +512,26 @@
   (let ((curbuf (current-buffer))
         (blink-matching-paren nil)
         (anzu--history (anzu--transform-from-to-history))
-        timer is-input)
+        is-input)
     (unwind-protect
         (minibuffer-with-setup-hook
             #'(lambda ()
-                (setq timer (run-with-idle-timer
-                             (max anzu-input-idle-delay 0.01)
-                             'repeat
-                             (lambda ()
-                               (anzu--clear-overlays curbuf nil nil)
-                               (with-selected-window (or (active-minibuffer-window)
-                                                         (minibuffer-window))
-                                 (anzu--check-minibuffer-input
-                                  curbuf beg end use-regexp overlay-limit))))))
+                (setq anzu--update-timer
+                      (run-with-idle-timer
+                       (max anzu-input-idle-delay 0.01)
+                       'repeat
+                       (lambda ()
+                         (anzu--clear-overlays curbuf nil nil)
+                         (with-selected-window (or (active-minibuffer-window)
+                                                   (minibuffer-window))
+                           (anzu--check-minibuffer-input
+                            curbuf beg end use-regexp overlay-limit))))))
           (prog1 (read-from-minibuffer (format "%s: " prompt)
                                        nil nil nil 'anzu--history nil t)
             (setq is-input t)))
-      (when timer
-        (cancel-timer timer)
-        (setq timer nil)
+      (when anzu--update-timer
+        (cancel-timer anzu--update-timer)
+        (setq anzu--update-timer nil)
         (unless is-input
           (goto-char beg))))))
 
@@ -540,7 +557,7 @@
       (add-to-history query-replace-from-history-variable from nil t)
       (when use-regexp
         (unless (anzu--validate-regexp from)
-          (error "'%s' is invalid regexp." from))
+          (error "'%s' is an invalid regular expression" from))
         (anzu--query-validate-from-regexp from))
       from)))
 
@@ -592,7 +609,9 @@
 (defsubst anzu--replaced-literal-string (ov replaced from)
   (let ((str (buffer-substring-no-properties
               (overlay-start ov) (overlay-end ov))))
-    (when (string-match (regexp-quote str) from)
+    ;; Needed to do `(string-match from str)' instead of `(string-match str from)',
+    ;; because lax whitespace means `from' can be a regexp.
+    (when (string-match from str)
       (replace-match replaced (not case-fold-search) t str))))
 
 (defun anzu--append-replaced-string (content buf beg end use-regexp overlay-limit from)
@@ -600,13 +619,14 @@
     (unless (string= content anzu--last-replace-input)
       (setq anzu--last-replace-input content)
       (with-current-buffer buf
-        (let ((case-fold-search (anzu--case-fold-search)))
+        (let ((case-fold-search (anzu--case-fold-search))
+              (pattern (anzu--convert-for-lax-whitespace from use-regexp)))
           (dolist (ov (anzu--overlays-in-range beg (min end overlay-limit)))
             (let ((replace-evaled
                    (if (not use-regexp)
-                       (anzu--replaced-literal-string ov content from)
+                       (anzu--replaced-literal-string ov content pattern)
                      (prog1 (anzu--evaluate-occurrence ov content replacements
-                                                       (not case-fold-search) from)
+                                                       (not case-fold-search) pattern)
                        (cl-incf replacements)))))
               (overlay-put ov 'after-string (anzu--propertize-to-string replace-evaled)))))))))
 
@@ -621,7 +641,7 @@
         (to-prompt (format "%s %s with: " prompt (query-replace-descr from)))
         (history-add-new-input nil)
         (blink-matching-paren nil)
-        timer is-input)
+        is-input)
     (setq anzu--last-replace-input "")
     (when anzu--outside-point
       (setq beg anzu--outside-point
@@ -630,22 +650,23 @@
     (unwind-protect
         (minibuffer-with-setup-hook
             #'(lambda ()
-                (setq timer (run-with-idle-timer
-                             (max anzu-input-idle-delay 0.01)
-                             'repeat
-                             (lambda ()
-                               (with-selected-window (or (active-minibuffer-window)
-                                                         (minibuffer-window))
-                                 (anzu--append-replaced-string
-                                  (minibuffer-contents)
-                                  curbuf beg end use-regexp overlay-limit from))))))
+                (setq anzu--update-timer
+                      (run-with-idle-timer
+                       (max anzu-input-idle-delay 0.01)
+                       'repeat
+                       (lambda ()
+                         (with-selected-window (or (active-minibuffer-window)
+                                                   (minibuffer-window))
+                           (anzu--append-replaced-string
+                            (minibuffer-contents)
+                            curbuf beg end use-regexp overlay-limit from))))))
           (prog1 (read-from-minibuffer to-prompt
                                        nil nil nil
                                        query-replace-from-history-variable nil t)
             (setq is-input t)))
-      (when timer
-        (cancel-timer timer)
-        (setq timer nil)
+      (when anzu--update-timer
+        (cancel-timer anzu--update-timer)
+        (setq anzu--update-timer nil)
         (unless is-input
           (goto-char orig-beg))))))
 
@@ -721,8 +742,8 @@
         (t nil)))
 
 (defun anzu--replace-backward-p (prefix)
-  ;; This variable is introduced at Emacs 24.4, I should fix this variable to
-  ;; version variable
+  ;; This variable was introduced in Emacs 24.4, I should fix this
+  ;; variable to version variable
   (and (boundp 'list-matching-lines-prefix-face)
        (and prefix (< prefix 0))))
 
@@ -831,31 +852,31 @@
 
 ;;;###autoload
 (defun anzu-query-replace-at-cursor ()
-  "Replace symbol at cursor with to-string."
+  "Replace the symbol at point."
   (interactive)
   (anzu--query-replace-common t :at-cursor t))
 
 ;;;###autoload
 (defun anzu-query-replace-at-cursor-thing ()
-  "Replace symbol at cursor within `anzu-replace-at-cursor-thing' area."
+  "Replace the thing at point, determined by variable `anzu-replace-at-cursor-thing'."
   (interactive)
   (anzu--query-replace-common t :at-cursor t :thing anzu-replace-at-cursor-thing))
 
 ;;;###autoload
 (defun anzu-query-replace (arg)
-  "anzu version of `query-replace'."
+  "Anzu version of `query-replace'."
   (interactive "p")
   (anzu--query-replace-common nil :prefix-arg arg))
 
 ;;;###autoload
 (defun anzu-query-replace-regexp (arg)
-  "anzu version of `query-replace-regexp'."
+  "Anzu version of `query-replace-regexp'."
   (interactive "p")
   (anzu--query-replace-common t :prefix-arg arg))
 
 ;;;###autoload
 (defun anzu-replace-at-cursor-thing ()
-  "anzu-query-replace-at-cursor-thing without query."
+  "Like `anzu-query-replace-at-cursor-thing', but without the query."
   (interactive)
   (let ((orig (point-marker)))
     (anzu--query-replace-common t
@@ -883,13 +904,13 @@
 
 ;;;###autoload
 (defun anzu-isearch-query-replace (arg)
-  "anzu version of `isearch-query-replace'."
+  "Anzu version of `isearch-query-replace'."
   (interactive "p")
   (anzu--isearch-query-replace-common nil arg))
 
 ;;;###autoload
 (defun anzu-isearch-query-replace-regexp (arg)
-  "anzu version of `isearch-query-replace-regexp'."
+  "Anzu version of `isearch-query-replace-regexp'."
   (interactive "p")
   (anzu--isearch-query-replace-common t arg))
 
